@@ -1,5 +1,5 @@
 /*
-Copyright (C) 1997-2005 Erwin Waterlander
+Copyright (C) 1997-2011 Erwin Waterlander
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -73,9 +73,9 @@ int stack_read(WcdStack ws,char *stackfilename)
 
 
 	/* open stack-file */
-	if  ((ws->maxsize <= 0)||((infile = fopen(stackfilename,"r")) == NULL))
+	if  ((ws->maxsize <= 0)||((infile = wcd_fopen(stackfilename,"r",1)) == NULL))
 	{
-		/* fprintf(stderr,"Wcd: error: Permission denied to read file %s\n",stackfilename); */
+		/* fprintf(stderr,"Wcd: error: Unable to read file %s\n",stackfilename); */
 		ws->lastadded = -1;
 		ws->current = -1;
 	}
@@ -224,23 +224,20 @@ int stack_write(WcdStack ws,char *stackfilename)
 		if ( (ptr = strrchr(path,DIR_SEPARATOR)) != NULL)
 		{
 			*ptr = '\0' ;
-			if (wcd_isdir(path) != 0) /* is it a dir */
+			if (wcd_isdir(path,1) != 0) /* is it a dir */
 			{
 #if (defined(UNIX) || defined(DJGPP) || defined(OS2))
 			m = S_IRWXU | S_IRWXG | S_IRWXO;
-				if (wcd_mkdir(path,m)!=0)
+				if (wcd_mkdir(path,m,0)==0)
 #else
-				if (wcd_mkdir(path)!=0)
+				if (wcd_mkdir(path,0)==0)
 #endif
-					fprintf(stderr,_("Wcd: error: Permission denied to create directory %s\n"), path);
-				else
 					fprintf(stderr,_("Wcd: creating directory %s\n"), path);
 			}
 		}
 
-		if ( (outfile = fopen(stackfilename,"w")) == NULL)
+		if ( (outfile = wcd_fopen(stackfilename,"w",0)) == NULL)
 		{
-			fprintf(stderr,_("Wcd: error: Write access to file %s denied.\n"),stackfilename);
 			return(0);
 		}
 		else
