@@ -8,7 +8,7 @@ Author: Erwin Waterlander
 ======================================================================
 = Copyright                                                          =
 ======================================================================
-Copyright (C) 2002-2009 Erwin Waterlander
+Copyright (C) 2002-2011 Erwin Waterlander
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -43,6 +43,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #include "wcddir.h"
 #include "match.h"
 #include <string.h>
+#include <errno.h>
 
 #if (defined(WIN32) && defined(WCD_UNICODE))
 #include <wchar.h>
@@ -64,6 +65,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #include "dosdir.h"
 #include <string.h>
 #include <windows.h>
+//#include <strsafe.h>
 #include "wcd.h"
 #include "display.h"
 
@@ -220,6 +222,65 @@ void wcd_getshares(char* path, nameset n)
 /* WIN32, not CYGWIN
    Use WIN32 API */
 
+
+void PrintError(DWORD dw) 
+{ 
+    /* Retrieve the system error message for the last-error code */
+
+    LPVOID lpMsgBuf;
+
+    FormatMessage(
+        FORMAT_MESSAGE_ALLOCATE_BUFFER | 
+        FORMAT_MESSAGE_FROM_SYSTEM |
+        FORMAT_MESSAGE_IGNORE_INSERTS,
+        NULL,
+        dw,
+        MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
+        (LPTSTR) &lpMsgBuf,
+        0, NULL );
+
+    /* Display the error message */
+
+    /* MessageBox(NULL, (LPCTSTR)lpMsgBuf, TEXT("Error"), MB_OK); */
+    fprintf(stderr, "%s\n",(LPCTSTR)lpMsgBuf);
+
+    LocalFree(lpMsgBuf);
+}
+
+
+
+/*
+   int wcd_isSharePath (char* path)
+   check if path is a possible UNC share path
+   like \\servername\sharename
+   RETURNS  1 if true
+   RETURNS  0 if false
+ */
+int wcd_isSharePath (char* path)
+{
+   char *ptr;
+
+   /* Assume the path has been fixed by wcd_fixpath(). Only forward slashes. */
+   if ((strlen(path) > 2) && (wcd_is_slash(*path)) && (wcd_is_slash(*(path+1))))
+   {
+      ptr = strchr(path+2,'/');
+      if (ptr == NULL)
+        return(0);
+      else
+      {
+        ptr = strchr(ptr+1,'/');
+        if (ptr == NULL)
+        {
+           return(1);
+        }
+        else
+           return(0);
+      }
+   }
+   else
+     return(0);
+}
+
 /* If WCD_UNICODE is defined we assume that all multi-byte
  * strings are encoded in UTF-8.
  */
@@ -227,6 +288,7 @@ void wcd_getshares(char* path, nameset n)
 char *wcd_getcwd(char *buf, int size)
 {
    BOOL err;
+   DWORD dw; 
 #ifdef WCD_UNICODE
    static wchar_t wstr[DD_MAXPATH];
 
@@ -239,14 +301,20 @@ char *wcd_getcwd(char *buf, int size)
 #endif
 
    if (err == 0)
-      return(NULL);  /* fail */
+   {
+     dw = GetLastError(); 
+     fprintf(stderr, "%s", _("Wcd: error: Unable to get current working directory: "));
+     PrintError(dw);
+     return(NULL);  /* fail */
+   }
    else
       return(buf);   /* success */
 }
 
-int wcd_chdir(char *buf)
+int wcd_chdir(char *buf, int quiet)
 {
    BOOL err;
+   DWORD dw; 
 #ifdef WCD_UNICODE
    static wchar_t wstr[DD_MAXPATH];
 
@@ -259,14 +327,23 @@ int wcd_chdir(char *buf)
 #endif
 
    if (err == 0)
+   {
+      if ( !quiet )
+      {
+         dw = GetLastError(); 
+         fprintf(stderr,_("Wcd: error: Unable to change to directory %s: "), buf);
+         PrintError(dw);
+      }
       return(1);   /* fail */
+   }
    else
       return(0);   /* success */
 }
 
-int wcd_mkdir(char *buf)
+int wcd_mkdir(char *buf, int quiet)
 {
    BOOL err;
+   DWORD dw; 
 #ifdef WCD_UNICODE
    static wchar_t wstr[DD_MAXPATH];
 
@@ -281,12 +358,21 @@ int wcd_mkdir(char *buf)
    if (err == TRUE)
       return(0);  /* success */
    else
-      return(1);  /* fail */
+   {
+     if ( !quiet )
+     {
+       dw = GetLastError(); 
+       fprintf(stderr,_("Wcd: error: Unable to create directory %s: "), buf);
+       PrintError(dw);
+     }
+     return(1);  /* fail */
+   }
 }
 
-int wcd_rmdir(char *buf)
+int wcd_rmdir(char *buf, int quiet)
 {
    BOOL err;
+   DWORD dw; 
 #ifdef WCD_UNICODE
    static wchar_t wstr[DD_MAXPATH];
 
@@ -301,7 +387,15 @@ int wcd_rmdir(char *buf)
    if (err == TRUE)
       return(0);  /* success */
    else
-      return(1);  /* fail */
+   {
+     if ( !quiet )
+     {
+       dw = GetLastError(); 
+       fprintf(stderr,_("Wcd: error: Unable to remove directory %s: "), buf);
+       PrintError(dw);
+     }
+     return(1);  /* fail */
+   }
 }
 
 /******************************************************************
@@ -318,26 +412,51 @@ int wcd_rmdir(char *buf)
  *       stat(path, &buf) ;
  *          if (S_ISDIR(buf.st_mode)) { ... }
  *
+ * See also bug report at http://sourceforge.net/projects/mingw/
+ *   http://sourceforge.net/tracker/?func=detail&atid=102435&aid=3304800&group_id=2435
+ *
  * - The function 'opendir()' from <dirent.h> works on all systems,
  *   also on Windows UNC paths as above, but not all compilers have 'dirent'
  *   included. E.g. LCC 3.8 and Open Watcom 1.3 don't have it.
  *
- * - Using 'wcd_chdir()' is a portable solution.
+ * - Using 'wcd_chdir()' is a portable solution to check \\servername\sharename.
  *
  ******************************************************************/
-int wcd_isdir(char *dir)
+int wcd_isdir(char *dir, int quiet)
 {
+   struct stat buf;
+   char *errstr;
    char tmp[DD_MAXDIR];
 
-   wcd_getcwd(tmp, sizeof(tmp)); /* remember current dir */
-
-   if (wcd_chdir(dir) == 0) /* just try to change to dir */
+   if (wcd_isSharePath(dir))
    {
-     wcd_chdir(tmp); /* go back */
-     return(0);
+      wcd_getcwd(tmp, sizeof(tmp)); /* remember current dir */
+
+      if (wcd_chdir(dir, quiet) == 0) /* just try to change to dir */
+      {
+        wcd_chdir(tmp, quiet); /* go back */
+        return(0);
+      }
+      else
+         return(-1);
+   } else {
+      if (stat(dir, &buf) == 0)
+      {
+         if (S_ISDIR(buf.st_mode))
+            return(0);
+         else
+            return(-1);
+      }
+      else
+      {
+         if (!quiet)
+         {
+           errstr = strerror(errno);
+           fprintf(stderr,"Wcd: %s: %s\n", dir, errstr);
+         }
+         return(-1);
+      }
    }
-   else
-      return(-1);
 }
 
 
@@ -347,16 +466,36 @@ int wcd_isdir(char *dir)
 
 #if defined(UNIX) || defined(DJGPP) || defined(__OS2__)
 
-int wcd_mkdir(char *buf, mode_t m)
+int wcd_mkdir(char *buf, mode_t m, int quiet)
 {
-   return(mkdir(buf, m));
+  char *errstr;
+  int err;
+
+  err = mkdir(buf, m);
+
+  if ( !quiet && err)
+  {
+    errstr = strerror(errno);
+    fprintf(stderr,_("Wcd: error: Unable to create directory %s: %s\n"), buf, errstr);
+  }
+  return(err);
 }
 
 #else
 
-int wcd_mkdir(char *buf)
+int wcd_mkdir(char *buf, int quiet)
 {
-   return(mkdir(buf));
+  char *errstr;
+  int err;
+
+  err = mkdir(buf);
+
+  if ( !quiet && err)
+  {
+    errstr = strerror(errno);
+    fprintf(stderr,_("Wcd: error: Unable to create directory %s: %s\n"), buf, errstr);
+  }
+  return(err);
 }
 
 #endif
@@ -390,7 +529,7 @@ char *replace_volume_path_HOME(char *buf, int size)
       if ((home = getenv("HOME")) != NULL )
       {
          getcwd(tmp, sizeof(tmp)); /* remember current dir */
-         if (wcd_chdir(home) == 0)
+         if (wcd_chdir(home,0) == 0)
          {
             if (getcwd(home_abs, sizeof(home_abs)) == NULL)
             {
@@ -423,7 +562,7 @@ char *replace_volume_path_HOME(char *buf, int size)
                   }
                }
             }
-         wcd_chdir(tmp); /* go back */
+         wcd_chdir(tmp,0); /* go back */
          } else {
             status = 5; /* fail */
          }
@@ -472,22 +611,49 @@ char *replace_volume_path_HOME(char *buf, int size)
 char *wcd_getcwd(char *buf, int size)
 {
    char *err;
+   char *errstr;
 
    err = getcwd(buf, size);
+   if ( err == NULL)
+   {
+     errstr = strerror(errno);
+     fprintf(stderr,_("Wcd: error: Unable to get current working directory: %s\n"), errstr);
+   }
 #ifdef UNIX
-   replace_volume_path_HOME(buf,size);
+   else
+     replace_volume_path_HOME(buf,size);
 #endif
    return(err);
 }
 
-int wcd_chdir(char *buf)
+int wcd_chdir(char *buf, int quiet)
 {
-   return(chdir(buf));
+  char *errstr;
+  int err;
+
+  err = chdir(buf);
+
+  if ( !quiet && err)
+  {
+    errstr = strerror(errno);
+    fprintf(stderr,_("Wcd: error: Unable to change to directory %s: %s\n"), buf, errstr);
+  }
+  return(err);
 }
 
-int wcd_rmdir(char *buf)
+int wcd_rmdir(char *buf, int quiet)
 {
-   return(rmdir(buf));
+  char *errstr;
+  int err;
+
+  err = rmdir(buf);
+
+  if ( !quiet && err)
+  {
+    errstr = strerror(errno);
+    fprintf(stderr,_("Wcd: error: Unable to remove directory %s: %s\n"), buf, errstr);
+  }
+  return(err);
 }
 
 /******************************************************************
@@ -499,14 +665,27 @@ int wcd_rmdir(char *buf)
  * returns 0 on success, -1 when it fails.
  *
  ******************************************************************/
-int wcd_isdir(char *dir)
+int wcd_isdir(char *dir, int quiet)
 {
    struct stat buf;
+   char *errstr;
 
-   if ((stat(dir, &buf) == 0) && S_ISDIR(buf.st_mode))
-      return(0);
+   if (stat(dir, &buf) == 0)
+   {
+      if (S_ISDIR(buf.st_mode))
+         return(0);
+      else
+         return(-1);
+   }
    else
+   {
+      if (!quiet)
+      {
+        errstr = strerror(errno);
+        fprintf(stderr,"Wcd: %s: %s\n", dir, errstr);
+      }
       return(-1);
+   }
 }
 
 
